@@ -278,13 +278,18 @@ set_pattributes(Head, Options) :-
     (   get_dict(dynamic, Options, true)
     ->  '$set_predicate_attribute'(Head, dynamic, true)
     ;   true
+    ),
+    (   get_dict(tshared, Options, true)
+    ->  '$set_predicate_attribute'(Head, tshared, true)
+    ;   true
     ).
 
+
 start_tabling(Wrapper, Worker) :-
     '$tbl_variant_table'(Wrapper, Trie, Status, Skeleton),
     (   Status == complete
     ->  '$idg_add_edge'(Trie),
-        '$tbl_answer_update_dl'(Trie, Skeleton)
+        trie_gen_compiled(Trie, Skeleton)
     ;   Status == fresh
     ->  '$tbl_create_subcomponent'(SCC, Trie),
         tdebug(user_goal(Wrapper, Goal)),
@@ -301,7 +306,7 @@ start_tabling(Wrapper, Worker) :-
     ;   Status == invalid
     ->  reeval(Trie),
         '$idg_add_edge'(Trie),
-        '$tbl_answer_update_dl'(Trie, Skeleton)
+        trie_gen_compiled(Trie, Skeleton)
     ;   % = run_follower, but never fresh and Status is a worklist
         shift(call_info(Skeleton, Status))
     ).
@@ -343,11 +348,11 @@ start_subsumptive_tabling(Wrapper, Worker) :-
 
 done_leader(complete, _SCC, Skeleton, Trie) :-
     !,
-    '$tbl_answer_update_dl'(Trie, Skeleton).
+    trie_gen_compiled(Trie, Skeleton).
 done_leader(final, SCC, Skeleton, Trie) :-
     !,
     '$tbl_free_component'(SCC),
-    '$tbl_answer_update_dl'(Trie, Skeleton).
+    trie_gen_compiled(Trie, Skeleton).
 done_leader(_,_,_,_).
 
 finished_leader(exit, _, _) :-
@@ -439,7 +444,7 @@ delim(Wrapper, Worker, WorkList, Delays) :-
 
 
 start_tabling(Wrapper, Worker, WrapperNoModes, ModeArgs) :-
-    '$tbl_variant_table'(WrapperNoModes, Trie, Status, _Skeleton),
+    '$tbl_moded_variant_table'(WrapperNoModes, Trie, Status, _Skeleton),
     (   Status == complete
     ->  '$idg_add_edge'(Trie),
         trie_gen(Trie, WrapperNoModes, ModeArgs)
@@ -935,6 +940,12 @@ table_options(incremental, Opts0, Opts1) :-
 table_options(dynamic, Opts0, Opts1) :-
     !,
     put_dict(dynamic, Opts0, true, Opts1).
+table_options(shared, Opts0, Opts1) :-
+    !,
+    put_dict(tshared, Opts0, true, Opts1).
+table_options(private, Opts0, Opts1) :-
+    !,
+    put_dict(tshared, Opts0, false, Opts1).
 table_options(Opt, _, _) :-
     '$domain_error'(table_option, Opt).
 
@@ -1282,7 +1293,7 @@ system:term_expansion((:- table(Preds)), Expansion) :-
     phrase(wrappers(Preds, M), Clauses),
     multifile_decls(Clauses, Directives0),
     sort(Directives0, Directives),
-    '$append'(Clauses, Directives, Expansion).
+    '$append'(Directives, Clauses, Expansion).
 
 multifile_decls([], []).
 multifile_decls([H0|T0], [H|T]) :-
@@ -1298,7 +1309,7 @@ multifile_decl(M:(Head :- _Body), (:- multifile(M:Name/Arity))) :-
 multifile_decl(M:Head, (:- multifile(M:Name/Arity))) :-
     !,
     functor(Head, Name, Arity).
-multifile_decl((Head :- body), (:- multifile(Name/Arity))) :-
+multifile_decl((Head :- _Body), (:- multifile(Name/Arity))) :-
     !,
     functor(Head, Name, Arity).
 multifile_decl(Head, (:- multifile(Name/Arity))) :-
