@@ -74,6 +74,8 @@ typedef struct
     code multifile[3];			/* S_MULTIFILE */
     code staticp[3];			/* S_STATIC */
     code wrapper[3];			/* S_WRAP */
+    code trie_gen2[3];			/* S_TRIE_GEN2 */
+    code trie_gen3[3];			/* S_TRIE_GEN3 */
   } supervisors;
 } PL_code_data_t;
 
@@ -175,6 +177,10 @@ struct PL_global_data
   { Table	modules;		/* atom --> module */
   } tables;
 
+  struct
+  { struct trie *variant_table;		/* Variant --> table */
+  } tabling;
+
   struct
   { Table	record_lists;		/* Available record lists */
   } recorded_db;
