@@ -61,6 +61,7 @@ typedef struct
   char *	topLevel;		/* toplevel goal */
   char *	initFile;		/* -f initialisation file */
   char *	systemInitFile;		/* -F initialisation file */
+  char *	config;			/* Show config info */
   opt_list     *scriptFiles;
   opt_list     *search_paths;		/* -p path */
   char *	pldoc_server;		/* --pldoc=Server */
@@ -68,6 +69,8 @@ typedef struct
   char *	saveclass;		/* Type of saved state */
   bool		silent;			/* -q: quiet operation */
   bool		traditional;		/* --traditional: no version 7 exts */
+  bool		nothreads;		/* --no-threads */
+  int		xpce;			/* --no-pce */
 #ifdef __WINDOWS__
   bool		win_app;		/* --win_app: be Windows application */
 #endif
