@@ -192,5 +192,8 @@ typedef struct idg_node
 
 COMMON(void)	clearThreadTablingData(PL_local_data_t *ld);
 COMMON(term_t)	init_delay_list(void);
+COMMON(void)	tbl_push_delay(atom_t atrie, Word wrapper,
+			       trie_node *answer ARG_LD);
+COMMON(int)	answer_is_conditional(trie_node *answer);
 
 #endif /*_PL_TABLING_H*/
