@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1999-2017, University of Amsterdam
+    Copyright (c)  1999-2019, University of Amsterdam
                               VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -218,9 +219,10 @@ extern counting_mutex _PL_mutexes[];	/* Prolog mutexes */
 #define L_UMUTEX       23
 #define L_INIT_ATOMS   24
 #define L_CGCGEN       25
+#define L_EVHOOK       26
 #ifdef __WINDOWS__
-#define L_DDE	       26
-#define L_CSTACK       27
+#define L_DDE	       27
+#define L_CSTACK       28
 #endif
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -449,7 +451,6 @@ COMMON(Atom**)	pl_atom_buckets_in_use(void);
 COMMON(Definition*)	predicates_in_use(void);
 COMMON(int)	pl_functor_table_in_use(FunctorTable functor_table);
 COMMON(int)	pl_kvs_in_use(KVS kvs);
-COMMON(void)	cgcActivatePredicate__LD(Definition def, gen_t gen ARG_LD);
 COMMON(gen_t)	pushPredicateAccess__LD(Definition def ARG_LD);
 COMMON(void)	popPredicateAccess__LD(Definition def ARG_LD);
 COMMON(size_t)	popNPredicateAccess__LD(size_t n ARG_LD);
